#ifndef COORD_HPP
#define COORD_HPP

#include "Common.hpp"
#include <gmpxx.h>

class Coord
{
public:
    Coord() :
        x(0), y(0) {}
    Coord(const mpq_class& xIn, const mpq_class& yIn) :
        x(xIn), y(yIn) {}
    Coord(mpq_class&& xIn, mpq_class&& yIn) :
        x(std::move(xIn)), y(std::move(yIn)) {}

    bool operator==(const Coord& other) const
    {
        return x == other.x && y == other.y;
    }

    bool operator!=(const Coord& other) const
    {
        return x != other.x || y != other.y;
    }

    bool operator<(const Coord& other) const
    {
        if (x < other.x) return true;
        if (other.x < x) return false;
        return y < other.y;
    }

public:
    mpq_class x;
    mpq_class y;
};

inline Coord operator+(const Coord& a, const Coord& b)
{
    return Coord(a.x + b.x, a.y + b.y);
}

inline Coord operator-(const Coord& a, const Coord& b)
{
    return Coord(a.x - b.x, a.y - b.y);
}

inline Coord operator*(const mpq_class& a, const Coord& b)
{
    return Coord(a * b.x, a * b.y);
}

inline Coord operator*(const Coord& a, const mpq_class& b)
{
    return Coord(a.x * b, a.y * b);
}

inline Coord operator/(const Coord& a, const mpq_class& b)
{
    return Coord(a.x / b, a.y / b);
}

inline mpq_class dot(const Coord& a, const Coord& b)
{
    return a.x * b.x + a.y * b.y;
}

inline mpq_class cross(const Coord& a, const Coord& b)
{
    return a.x * b.y - a.y * b.x;
}

#endif
