#include "Geometry.hpp"
#include "Problem.hpp"
#include <gmpxx.h>

Dir getPolygonDir(const Polygon& polygon)
{
    mpq_class sum(0);
    auto& vertices = polygon.m_vertices;
    const Coord* pPrevCoord = &vertices.back();
    for (auto& coord : vertices)
    {
        auto& prevCoord = *pPrevCoord;
        sum += prevCoord.x * coord.y;
        sum -= prevCoord.y * coord.x;
        pPrevCoord = &coord;
    }

    return (sum > 0) ? Dir::CCW : Dir::CW;
}

bool isPolygonConvex(const Polygon& polygon)
{
    bool gotDir = false;
    Dir dir = Dir::CCW;
    auto& vertices = polygon.m_vertices;
    const Coord* pCoord1 = &vertices[vertices.size() - 2];
    const Coord* pCoord2 = &vertices[vertices.size() - 1];
    for (auto& coord3 : vertices)
    {
        auto& coord1 = *pCoord1;
        auto& coord2 = *pCoord2;
        mpq_class dx1 = coord2.x - coord1.x;
        mpq_class dy1 = coord2.y - coord1.y;
        mpq_class dx2 = coord3.x - coord2.x;
        mpq_class dy2 = coord3.y - coord2.y;
        mpq_class cross = dx1 * dy2 - dx2 * dy1;
        if (cross > 0)
        {
            if (!gotDir)
            {
                gotDir = true;
                dir = Dir::CCW;
            }
            else if (dir != Dir::CCW)
            {
                return false;
            }
        }
        else
        {
            if (!gotDir)
            {
                gotDir = true;
                dir = Dir::CW;
            }
            else if (dir != Dir::CW)
            {
                return false;
            }
        }
        pCoord1 = pCoord2;
        pCoord2 = &coord3;
    }

    return true;
}

Coord project(const Coord& a, const Coord& b)
{
    return b * dot(a, b) / dot(b, b);
}

Coord reflect(const Coord& p, const Coord& q, const Coord& a)
{
    Coord q_p = q - p;
    Coord a_p = a - p;
    Coord proj = project(a_p, q_p);
    Coord b_p = 2 * proj - a_p;
    return b_p + p;
}

bool intersectLines(const Coord& orig1,
                    const Coord& dir1,
                    const Coord& orig2,
                    const Coord& dir2,
                    mpq_class* pParam1,
                    mpq_class* pParam2)
{
    mpq_class dx = cross(dir1, dir2);
    if (dx == 0)
    {
        // Lines are parallel or coincident
        return false;
    }
    Coord ds = orig2 - orig1;
    *pParam1 = cross(ds, dir2) / dx;
    *pParam2 = cross(ds, dir1) / dx;
    return true;
}
