#ifndef PROTOCOL_HPP
#define PROTOCOL_HPP

#include "Common.hpp"

class SnapshotRec
{
public:
    SnapshotRec() : m_time(0) {}

    uint64_t m_time;
    std::string m_hash;
};

class Ranking
{
public:
    Ranking() : m_resemblance(0.0), m_solutionSize(0) {}

    double m_resemblance;
    uint32_t m_solutionSize;
};

class ProblemRec
{
public:
    ProblemRec() :
        m_time(0), m_solutionSize(0), m_id(0), m_problemSize(0) {}

    std::vector<Ranking> m_rankings;
    uint64_t m_time;
    uint32_t m_solutionSize;
    uint32_t m_id;
    std::string m_owner;
    uint32_t m_problemSize;
    std::string m_hash;
};

class LeaderboardRec
{
public:
    LeaderboardRec() : m_score(0.0) {}

    std::string m_username;
    double m_score;
};

class User
{
public:
    std::string m_username;
    std::string m_displayName;
};

class Snapshot
{
public:
    std::vector<ProblemRec> m_problemRecs;
    uint64_t m_time;
    std::vector<LeaderboardRec> m_leaderboardRecs;
    std::vector<User> m_users;
};

class SolutionResult
{
public:
    SolutionResult() : m_id(0), m_resemblance(0.0), m_solutionSize(0) {}

    uint32_t m_id;
    double m_resemblance;
    std::string m_solutionHash;
    uint32_t m_solutionSize;
};

class ProblemResult
{
public:
    ProblemResult() :
        m_id(0), m_time(0.0), m_solutionSize(0), m_problemSize(0) {}

    uint32_t m_id;
    uint64_t m_time;
    std::string m_solutionHash;
    uint32_t m_solutionSize;
    std::string m_problemHash;
    uint32_t m_problemSize;
};

namespace Protocol
{
    void init();
    void cleanup();

    bool helloWorld(std::string* pMsg = nullptr);
    bool getSnapshotList(std::vector<SnapshotRec>* pSnapshotRecs,
                         std::string* pMsg = nullptr);
    bool getSnapshot(const std::string& snapshotHash,
                     Snapshot* pSnapshot,
                     std::string* pMsg = nullptr);
    bool getProblem(const std::string& problemHash,
                    std::string* pProblem,
                    std::string* pMsg = nullptr);
    bool submitSolution(uint32_t id,
                        const std::string& solution,
                        SolutionResult* pResult,
                        std::string* pMsg = nullptr);
    bool submitProblem(uint64_t time,
                       const std::string& solution,
                       ProblemResult* pResult,
                       std::string* pMsg = nullptr);
}

#endif
