#include "Solution.hpp"
#include "StringUtils.hpp"
#include <gmpxx.h>

using std::string;

bool formatSolution(const Solution& solution,
                    string* pSolutionData,
                    uint32_t* pSize,
                    string* pMsg)
{
    string dummyMsg;
    if (!pMsg)
    {
        pMsg = &dummyMsg;
    }

    string dummyData;
    auto& data = pSolutionData ? *pSolutionData : dummyData;
    data.reserve(1000);

    data += strprintf("%" PRIuZ "\n", solution.m_sourceCoords.size());
    for (auto& coord : solution.m_sourceCoords)
    {
        data += coord.x.get_str();
        data += ",";
        data += coord.y.get_str();
        data += "\n";
    }

    data += strprintf("%" PRIuZ "\n", solution.m_facets.size());
    for (auto& facet : solution.m_facets)
    {
        data += strprintf("%" PRIuZ "", facet.size());
        for (uint32_t idx : facet)
        {
            if (idx >= (uint32_t)solution.m_sourceCoords.size())
            {
                *pMsg = "Bad facet vertex";
                return false;
            }
            data += strprintf(" %" PRIu32 "", idx);
        }
        data += "\n";
    }

    if (solution.m_destCoords.size() != solution.m_sourceCoords.size())
    {
        *pMsg = "Different numbers of source and destination coordinates";
        return false;
    }
    for (auto& coord : solution.m_destCoords)
    {
        data += coord.x.get_str();
        data += ",";
        data += coord.y.get_str();
        data += "\n";
    }

    uint32_t size = 0;
    for (char ch : data)
    {
        if (ch != ' ' && ch != '\r' && ch != '\n')
        {
            size++;
        }
    }

    /*
    if (size > 5000)
    {
        *pMsg = strprintf("Size %" PRIu32 " exceeds limit", size);
        return false;
    }
    */

    if (pSize)
    {
        *pSize = size;
    }

    return true;
}
