#ifndef SOLVER_HPP
#define SOLVER_HPP

#include "Common.hpp"
#include "Coord.hpp"
#include "Trf.hpp"

class Problem;
class Segment;
class Solution;

class ShapeVertexEdge
{
public:
    ShapeVertexEdge() :
        m_edgeId((uint32_t)-1),
        m_vertexIdx((uint32_t)-1)
    {
    }

    uint32_t m_edgeId;
    uint32_t m_vertexIdx;
};

class ShapeVertex
{
public:
    Coord m_coord;
    std::vector<ShapeVertexEdge> m_edges;
};

class ShapeEdgeVertex
{
public:
    ShapeEdgeVertex() :
        m_vertexId((uint32_t)-1),
        m_edgeIdx((uint32_t)-1)
    {
    }

    uint32_t m_vertexId;
    uint32_t m_edgeIdx;
};

class ShapeEdgeRegion
{
public:
    ShapeEdgeRegion() :
        m_regionId((uint32_t)-1),
        m_edgeIdx((uint32_t)-1)
    {
    }

    uint32_t m_regionId;
    uint32_t m_edgeIdx;
};

class ShapeEdge
{
public:
    ShapeEdgeVertex m_vertices[2];
    ShapeEdgeRegion m_regions[2];
};

enum class ShapeRegionType : uint32_t
{
    Interior,
    Exterior
};

class ShapeRegionEdge
{
public:
    ShapeRegionEdge() :
        m_edgeId((uint32_t)-1),
        m_regionIdx((uint32_t)-1)
    {
    }

    uint32_t m_edgeId;
    uint32_t m_regionIdx;
};

class ShapeRegion
{
public:
    ShapeRegion() :
        m_type(ShapeRegionType::Interior),
        m_flipped(0),
        m_used(0),
        m_partnerRegionId((uint32_t)-1) {}

    ShapeRegionType m_type;
    uint32_t m_flipped;
    uint32_t m_used;
    uint32_t m_partnerRegionId;
    Trf m_trf;
    Trf m_invTrf;
    std::vector<ShapeRegionEdge> m_edges;
    //std::vector<ShapeRegionVertex> m_vertices;
};

class Shape
{
public:
    uint32_t findVertex(const Coord& vertex) const;

    std::vector<ShapeVertex> m_vertices;
    std::vector<ShapeEdge> m_edges;
    std::vector<ShapeRegion> m_regions;
};

void splitSegments(const std::vector<Segment>& segments,
                   std::vector<Segment>* pSplitSegments);

void problemToShape(const Problem& problem,
                    Shape* pShape);

bool solve(const Problem& problem,
           uint64_t seed = 1,
           uint32_t maxDurationMS = 0,
           Solution* pSolution = nullptr,
           std::string* pSolutionData = nullptr,
           uint32_t* pSolutionSize = nullptr,
           uint64_t* pNodeCount = nullptr);

#endif
