#ifndef TRF_HPP
#define TRF_HPP

#include "Common.hpp"
#include "Coord.hpp"

class Trf
{
public:
    Trf() :
        a(1), b(0), c(0), d(1), x(0), y(0) {}
    Trf(const mpq_class& aIn,
        const mpq_class& bIn,
        const mpq_class& cIn,
        const mpq_class& dIn,
        const mpq_class& xIn,
        const mpq_class& yIn) :
        a(aIn), b(bIn), c(cIn), d(dIn), x(xIn), y(yIn) {}
    Trf(mpq_class&& aIn,
        mpq_class&& bIn,
        mpq_class&& cIn,
        mpq_class&& dIn,
        mpq_class&& xIn,
        mpq_class&& yIn) :
        a(std::move(aIn)),
        b(std::move(bIn)),
        c(std::move(cIn)),
        d(std::move(dIn)),
        x(std::move(xIn)),
        y(std::move(yIn)) {}

    bool operator==(const Trf& other) const
    {
        return
            a == other.a &&
            b == other.b &&
            c == other.c &&
            d == other.d &&
            x == other.x &&
            y == other.y;
    }

    bool operator!=(const Trf& other) const
    {
        return
            a != other.a ||
            b != other.b ||
            c != other.c ||
            d != other.d ||
            x != other.x ||
            y != other.y;
    }

    mpq_class a;
    mpq_class b;
    mpq_class c;
    mpq_class d;
    mpq_class x;
    mpq_class y;
};

inline Coord operator*(const Trf& a, const Coord& b)
{
    return Coord(a.a * b.x + a.b * b.y + a.x,
                 a.c * b.x + a.d * b.y + a.y);
}

inline Trf operator*(const Trf& a, const Trf& b)
{
    return Trf(a.a * b.a + a.b * b.c,
               a.a * b.b + a.b * b.d,
               a.c * b.a + a.d * b.c,
               a.c * b.b + a.d * b.d,
               a.a * b.x + a.b * b.y + a.x,
               a.c * b.x + a.d * b.y + a.y);
}

inline Trf invert(const Trf& a)
{
    mpq_class det = a.a * a.d - a.b * a.c;
    Trf result = Trf(a.d / det,
                     -a.b / det,
                     -a.c / det,
                     a.a / det,
                     (a.b * a.y - a.d * a.x) / det,
                     (a.c * a.x - a.a * a.y) / det);

    Trf identity1 = a * result;
    Trf identity2 = result * a;
    if (identity1.a != 1 ||
        identity1.b != 0 ||
        identity1.c != 0 ||
        identity1.d != 1 ||
        identity1.x != 0 ||
        identity1.y != 0 ||
        identity2.a != 1 ||
        identity2.b != 0 ||
        identity2.c != 0 ||
        identity2.d != 1 ||
        identity2.x != 0 ||
        identity2.y != 0)
    {
        printf("invert failed\n");
        printf("%s\n", identity1.a.get_str().c_str());
        printf("%s\n", identity1.b.get_str().c_str());
        printf("%s\n", identity1.c.get_str().c_str());
        printf("%s\n", identity1.d.get_str().c_str());
        printf("%s\n", identity1.x.get_str().c_str());
        printf("%s\n", identity1.y.get_str().c_str());
        exit(1);
    }
    return result;
}

inline Trf getMirrorTrf(const Coord& origin, const Coord& dir)
{
    mpq_class dirx2 = dir.x * dir.x;
    mpq_class diry2 = dir.y * dir.y;
    mpq_class len2 = dirx2 + diry2;
    mpq_class val1 = (dirx2 - diry2) / len2;
    mpq_class val2 = (2 * dir.x * dir.y) / len2;
    return Trf(val1,
               val2,
               val2,
               -val1,
               origin.x - (val1 * origin.x + val2 * origin.y),
               origin.y - (val2 * origin.x - val1 * origin.y));
}

#endif
