#include "Common.hpp"
#include "Protocol.hpp"
#include "Cleanup.hpp"
#include "StringUtils.hpp"

using std::string;
using std::vector;

int main(int argc, char *argv[])
{
    Protocol::init();
    Cleanup cleanupProtocol([](){ Protocol::cleanup(); });

    string msg;

    vector<SnapshotRec> snapshotRecs;
    if (!Protocol::getSnapshotList(&snapshotRecs, &msg))
    {
        printf("getSnapshotList failed: %s\n", msg.c_str());
        return 1;
    }

    if (snapshotRecs.empty())
    {
        printf("No snapshots found\n");
        return 1;
    }

    printf("Found %" PRIuZ " snapshots\n", snapshotRecs.size());

    uint64_t snapshotTime = 0;
    string snapshotHash;
    for (auto& snapshotRec : snapshotRecs)
    {
        /*
        printf("%" PRIu64 ": %s\n",
               snapshotRec.m_time,
               snapshotRec.m_hash.c_str());
        */
        if (snapshotRec.m_time > snapshotTime)
        {
            snapshotTime = snapshotRec.m_time;
            snapshotHash = snapshotRec.m_hash;
        }
    }

    if (snapshotTime == 0)
    {
        printf("Newest snapshot not found\n");
        return 1;
    }

    /*
    printf("Snapshot time %" PRIu64 ", hash %s\n",
           snapshotTime,
           snapshotHash.c_str());
    */

    uint64_t contestTime = snapshotTime - 1470355200;
    uint64_t contestHours = contestTime / 3600;
    uint64_t contestMinutes = (contestTime / 60) % 60;
    uint64_t contestSeconds = contestTime % 60;
    printf("Snapshot time: %" PRIu64 ":%02" PRIu64 ":%02" PRIu64 "\n",
           contestHours, contestMinutes, contestSeconds);

    Snapshot snapshot;
    if (!Protocol::getSnapshot(snapshotHash, &snapshot, &msg))
    {
        printf("getSnapshot failed: %s\n", msg.c_str());
        return 1;
    }

    printf("Found %" PRIuZ " problems\n", snapshot.m_problemRecs.size());

    for (auto& problemRec : snapshot.m_problemRecs)
    {
        string fileName = strprintf("../problems/%" PRIu32 ".txt",
                                    problemRec.m_id);
        FILE* f = fopen(fileName.c_str(), "rb");
        if (f)
        {
            fclose(f);
            continue;
        }

        printf("Fetching problem %" PRIu32 "... ", problemRec.m_id);
        fflush(stdout);

        string problem;
        if (!Protocol::getProblem(problemRec.m_hash, &problem, &msg))
        {
            printf("Error!\n");
            printf("%s\n", msg.c_str());
            return 1;
        }

        printf("done.\n");

        f = fopen(fileName.c_str(), "wb");
        if (!f)
        {
            printf("Error opening problem file for write\n");
            return 1;
        }

        if (fwrite(problem.data(), 1, problem.size(), f) != problem.size())
        {
            fclose(f);
            printf("Error writing to problem file\n");
            return 1;
        }

        fclose(f);

        //printf("Stopping early for now\n");
        //break;
    }

    return 0;
}
