#include "Common.hpp"
#include "Protocol.hpp"
#include "Cleanup.hpp"
#include "Solution.hpp"
#include "Challenge.hpp"
#include "StringUtils.hpp"

using std::string;
using std::vector;

int main(int argc, char *argv[])
{
    Protocol::init();
    Cleanup cleanupProtocol([](){ Protocol::cleanup(); });

    string msg;

#if 1
    /*
    if (Protocol::helloWorld(&msg))
    {
        printf("helloWorld succeeded\n");
    }
    else
    {
        printf("helloWorld failed: %s\n", msg.c_str());
    }
    */

    vector<SnapshotRec> snapshotRecs;
    if (!Protocol::getSnapshotList(&snapshotRecs, &msg))
    {
        printf("getSnapshotList failed: %s\n", msg.c_str());
        return 1;
    }

    uint64_t snapshotTime = 0;
    string snapshotHash;
    for (auto& snapshotRec : snapshotRecs)
    {
        /*
        printf("%" PRIu64 ": %s\n",
               snapshotRec.m_time,
               snapshotRec.m_hash.c_str());
        */
        if (snapshotRec.m_time > snapshotTime)
        {
            snapshotTime = snapshotRec.m_time;
            snapshotHash = snapshotRec.m_hash;
        }
    }

    if (snapshotTime == 0)
    {
        printf("No snapshots found\n");
        return 1;
    }

    Snapshot snapshot;
    if (!Protocol::getSnapshot(snapshotHash, &snapshot, &msg))
    {
        printf("getSnapshot failed: %s\n", msg.c_str());
        return 1;
    }

    printf("Problems:\n");
    for (auto& problemRec : snapshot.m_problemRecs)
    {
        bool firstRanking = true;
        uint32_t perfectCount = 0;
        double bestResemblance = 0;
        uint32_t bestPerfectSize = 0;
        for (auto& ranking : problemRec.m_rankings)
        {
            if (ranking.m_resemblance == 1.0)
            {
                if (perfectCount == 0 ||
                    ranking.m_solutionSize < bestPerfectSize)
                {
                    bestPerfectSize = ranking.m_solutionSize;
                }
                perfectCount++;
            }
            if (firstRanking ||
                ranking.m_resemblance > bestResemblance)
            {
                bestResemblance = ranking.m_resemblance;
                firstRanking = false;
            }
        }

        printf("\n");
        printf("  ID: %" PRIu32 "\n", problemRec.m_id);
        printf("  Owner: %s\n", problemRec.m_owner.c_str());
        printf("  Rankings: %" PRIuZ "\n", problemRec.m_rankings.size());
        printf("     Perfect count: %" PRIu32 "\n", perfectCount);
        printf("     Best resemblance: %f\n", bestResemblance);
        printf("     Best perfect size: %" PRIu32 "\n", bestPerfectSize);
        printf("  Time: %" PRIu64 "\n", problemRec.m_time);
        printf("  Solution size: %" PRIu32 "\n", problemRec.m_solutionSize);
        printf("  Problem size: %" PRIu32 "\n", problemRec.m_problemSize);
        printf("  Hash: %s\n", problemRec.m_hash.c_str());
    }
    printf("\n");

    printf("Time: %" PRIu64 "\n", snapshot.m_time);
    printf("\n");

    printf("Leaderboard:\n");
    for (auto& leaderboardRec : snapshot.m_leaderboardRecs)
    {
        printf("  %s: %0.12f\n",
               leaderboardRec.m_username.c_str(),
               leaderboardRec.m_score);
    }
    printf("\n");

    printf("Users:\n");
    for (auto& user : snapshot.m_users)
    {
        printf("  %s: %s\n",
               user.m_username.c_str(),
               user.m_displayName.c_str());
    }
    printf("\n");
    return 0;
#endif

    Solution solution;

#if 0
    solution.m_sourceCoords.push_back(Coord(0, 0));
    solution.m_sourceCoords.push_back(Coord(1, 0));
    solution.m_sourceCoords.push_back(Coord(1, 1));
    solution.m_sourceCoords.push_back(Coord(0, 1));

    solution.m_facets.resize(1);
    solution.m_facets[0].push_back(0);
    solution.m_facets[0].push_back(1);
    solution.m_facets[0].push_back(2);
    solution.m_facets[0].push_back(3);

    solution.m_destCoords.push_back(Coord(0, 0));
    solution.m_destCoords.push_back(Coord(1, 0));
    solution.m_destCoords.push_back(Coord(1, 1));
    solution.m_destCoords.push_back(Coord(0, 1));
#endif

#if 1
    makeChallenge(223, &solution);
#endif

    string solutionData;
    uint32_t solutionSize = 0;
    if (!formatSolution(solution, &solutionData, &solutionSize, &msg))
    {
        printf("%s\n", msg.c_str());
        return 1;
    }

    printf("%s", solutionData.c_str());
    printf("Solution size = %" PRIu32 "\n", solutionSize);

    if (solutionSize > 5000)
    {
        printf("Size %" PRIu32 " exceeds limit", solutionSize);
        return 1;
    }

#if 0
    SolutionResult result;
    if (!Protocol::submitSolution(1, solutionData, &result, &msg))
    {
        printf("%s\n", msg.c_str());
        return 1;
    }

    printf("id = %" PRIu32 "\n", result.m_id);
    printf("resemblance = %f\n", result.m_resemblance);
    printf("solution hash = %s\n", result.m_solutionHash.c_str());
    printf("solution size = %" PRIu32 "\n", result.m_solutionSize);
#endif

#if 0
    ProblemResult problemResult;
    uint64_t publishTime = 1470441600 + 3600 * 45;
    if (!Protocol::submitProblem(publishTime, solutionData, &problemResult, &msg))
    {
        printf("%s\n", msg.c_str());
        return 1;
    }

    printf("id = %" PRIu32 "\n", problemResult.m_id);
    printf("publish time = %" PRIu64 "\n", problemResult.m_time);
    printf("solution hash = %s\n", problemResult.m_solutionHash.c_str());
    printf("solution size = %" PRIu32 "\n", problemResult.m_solutionSize);
    printf("problem hash = %s\n", problemResult.m_problemHash.c_str());
    printf("problem size = %" PRIu32 "\n", problemResult.m_problemSize);

    {
        string problem;
        if (!Protocol::getProblem(problemResult.m_problemHash,
                                  &problem,
                                  &msg))
        {
            printf("%s\n", msg.c_str());
            return 1;
        }
        string fileName = strprintf("../challengeproblems/%" PRIu32 ".txt",
                                    problemResult.m_id);
        FILE* f = fopen(fileName.c_str(), "wb");
        if (!f)
        {
            printf("Error opening challenge file for write\n");
            return 1;
        }

        if (fwrite(problem.data(), 1, problem.size(), f) != problem.size())
        {
            fclose(f);
            printf("Error writing to challenge file\n");
            return 1;
        }

        fclose(f);
    }
#endif
    
    return 0;
}
