#include "Common.hpp"
#include "Graphics.hpp"
#include "Problem.hpp"
#include "Geometry.hpp"
#include "StringUtils.hpp"
#include "Solver.hpp"
#include <gmpxx.h>

using std::string;
using std::vector;

int main(int argc, char *argv[])
{
    if (argc != 1 && argc != 2)
    {
        printf("Usage: showproblem [<id>]\n");
        return 1;
    }

    uint32_t id = 1;
    if (argc == 2)
    {
        id = (uint32_t)atol(argv[1]);
    }

    Problem problem;
    bool haveProblem = false;

    string loadMsg;

    //vector<Segment> segments;
    Shape shape;

    mpq_class minX;
    mpq_class minY;
    mpq_class maxX;
    mpq_class maxY;
    mpq_class dX;
    mpq_class dY;
    mpq_class dMax;

    auto reload = [&]()
    {
        if (!loadProblem(id, false, &problem, &loadMsg))
        {
            haveProblem = false;
            return;
        }

        //splitSegments(problem.m_segments, &segments);
        problemToShape(problem, &shape);

        haveProblem = true;

        bool firstCoord = true;

        auto processCoord = [&](const Coord& coord)
        {
            if (firstCoord)
            {
                firstCoord = false;
                minX = coord.x;
                maxX = coord.x;
                minY = coord.y;
                maxY = coord.y;
            }
            else
            {
                if (coord.x < minX) minX = coord.x;
                if (coord.x > maxX) maxX = coord.x;
                if (coord.y < minY) minY = coord.y;
                if (coord.y > maxY) maxY = coord.y;
            }
        };

        for (auto& polygon : problem.m_polygons)
        {
            for (auto& vertex : polygon.m_vertices)
            {
                processCoord(vertex);
            }
        }
        for (auto& edge : shape.m_edges)
        {
            processCoord(shape.m_vertices[edge.m_vertices[0].m_vertexId].m_coord);
            processCoord(shape.m_vertices[edge.m_vertices[1].m_vertexId].m_coord);
        }

        dX = maxX - minX;
        dY = maxY - minY;
        dMax = (dX > dY) ? dX : dY;
    };

    reload();

    bool destroyed = false;
    Graphics gr;

    auto onDestroy = [&]()
    {
        destroyed = true;
    };
    gr.setOnDestroy(onDestroy);

    auto onPaint = [&]()
    {
        gr.beginPaint(true);

        gr.fillRect(0, 0, gr.getWidth(), gr.getHeight(), 0x000000);

        if (haveProblem)
        {
#if 1
            for (uint32_t iDir = 0; iDir < 2; iDir++)
            {
                Dir dir = (Dir)iDir;
                for (auto& polygon : problem.m_polygons)
                {
                    if (getPolygonDir(polygon) != dir)
                    {
                        continue;
                    }
                    auto& vertices = polygon.m_vertices;
                    vector<Point> points(vertices.size());
                    for (uint32_t iVertex = 0; iVertex < (uint32_t)vertices.size(); iVertex++)
                    {
                        auto& vertex = vertices[iVertex];
                        mpq_class screenX = (vertex.x - minX) / dMax * 1000;
                        points[iVertex].x = (int32_t)screenX.get_d();
                        mpq_class screenY = (vertex.y - minY) / dMax * 1000;
                        points[iVertex].y = 1000 - (int32_t)screenY.get_d();
                    }
                    gr.fillPolygon(&points[0],
                                   (uint32_t)vertices.size(),
                                   dir == Dir::CCW ? 0xffffff : 0x000000);
                }
            }
#endif

#if 0
            uint32_t regionColor = 0x000000;
            for (auto& region : shape.m_regions)
            {
                if (region.m_type == ShapeRegionType::Exterior)
                {
                    continue;
                }

                vector<Point> points;
                for (auto& re : region.m_edges)
                {
                    auto& edge = shape.m_edges[re.m_edgeId];
                    auto& ev = edge.m_vertices[re.m_regionIdx];
                    auto& vertex = shape.m_vertices[ev.m_vertexId];
                    auto& coord = vertex.m_coord;

                    points.emplace_back();
                    auto& point = points.back();

                    mpq_class screenX = (coord.x - minX) / dMax * 1000;
                    point.x = (int32_t)screenX.get_d();
                    mpq_class screenY = (coord.y - minY) / dMax * 1000;
                    point.y = 1000 - (int32_t)screenY.get_d();
                }

                regionColor = (regionColor + 0x13476f) & 0xffffff;
                gr.fillPolygon(&points[0],
                               (uint32_t)points.size(),
                               regionColor);
            }
#endif

            //uint32_t segmentColor = 0x000000;

            gr.setLineWidth(3);
            for (auto& edge : shape.m_edges)
            {
                auto& segmentStart = shape.m_vertices[edge.m_vertices[0].m_vertexId].m_coord;
                auto& segmentEnd = shape.m_vertices[edge.m_vertices[1].m_vertexId].m_coord;

                mpq_class screenXstart = (segmentStart.x - minX) / dMax * 1000;
                int32_t startX = (int32_t)screenXstart.get_d();
                mpq_class screenYstart = (segmentStart.y - minY) / dMax * 1000;
                int32_t startY = 1000 - (int32_t)screenYstart.get_d();

                mpq_class screenXend = (segmentEnd.x - minX) / dMax * 1000;
                int32_t endX = (int32_t)screenXend.get_d();
                mpq_class screenYend = (segmentEnd.y - minY) / dMax * 1000;
                int32_t endY = 1000 - (int32_t)screenYend.get_d();

                //segmentColor = (segmentColor + 0x13476f) & 0xffffff;
                //gr.drawLine(startX, startY, endX, endY, segmentColor);

                int32_t midX = (startX + endX) / 2;
                int32_t midY = (startY + endY) / 2;

                uint32_t startColor = (0x200000 * edge.m_vertices[0].m_edgeIdx) & 0xffffff;
                uint32_t endColor = (0x200000 * edge.m_vertices[1].m_edgeIdx) & 0xffffff;
                gr.drawLine(startX, startY, midX, midY, startColor);
                gr.drawLine(midX, midY, endX, endY, endColor);
            }
        }

        gr.setFont(FontSpacing::Monospace, FontFace::SansSerif,
                   18, false, false);
        string status = strprintf("Problem %" PRIu32 "", id);
        if (!haveProblem)
        {
            status += ": ";
            status += loadMsg;
        }
        gr.drawText(10, 10,
                    status.c_str(),
                    0x00cccc, HAlign::Left, VAlign::Top);

        gr.endPaint();
    };
    gr.setOnPaint(onPaint);

    auto onKey = [&](Key key)
    {
        if (key == Key::Q)
        {
            gr.destroy();
        }

        if (key == Key::Right)
        {
            do
            {
                id++;
                if (id == 10001)
                {
                    id = 1;
                }
                reload();
            }
            while (!haveProblem);
            gr.requestPaint();
        }

        if (key == Key::Left)
        {
            do
            {
                id--;
                if (id == 0)
                {
                    id = 10000;
                }
                reload();
            }
            while (!haveProblem);
            gr.requestPaint();
        }
    };
    gr.setOnKey(onKey);

    string msg;
    if (!gr.create(1001, 1001, "Origami", &msg))
    {
        fprintf(stderr, "%s\n", msg.c_str());
        return 1;
    }

    while (Graphics::update(true))
    {
        if (destroyed)
        {
            break;
        }
    }

    return 0;
}
