#include "Common.hpp"
#include "Problem.hpp"
#include "Solver.hpp"
#include "Solution.hpp"
#include "Protocol.hpp"
#include "StringUtils.hpp"
#include "TimeUtils.hpp"
#include "ConvexSolver.hpp"
#include "ApproxSolver.hpp"
#include <time.h>

using std::string;

int main(int argc, char *argv[])
{
    if (argc != 1 && argc != 2 && argc != 3)
    {
        printf("Usage: solve [<id> [-f]]\n");
        return 1;
    }

    uint32_t startId = 1;
    if (argc >= 2)
    {
        startId = (uint32_t)atol(argv[1]);
    }

    bool force = false;
    if (argc >= 3)
    {
        force = true;
    }

    //uint64_t seed = 2;
    uint64_t seed = (uint64_t)time(nullptr);

    string msg;

    for (uint32_t id = startId; id <= 10000; id++)
    {
        if (id == 10000)
        {
            id = 1;
        }

        if (id == 581 ||
            id == 1082 ||
            id == 1239 ||
            id == 1460 ||
            id == 1492 ||
            id == 1843 ||
            id == 1875 ||
            id == 1885 ||
            id == 1899 ||
            id == 1933 ||
            id == 1941 ||
            id == 1952 ||
            id == 1965 ||
            id == 1971 ||
            id == 1972 ||
            id == 1973 ||
            id == 1977 ||
            id == 1979 ||
            id == 1980 ||
            id == 1982 ||
            id == 1984 ||
            id == 1987 ||
            id == 1989 ||
            id == 1991 ||
            id == 1999 ||
            id == 4637 ||
            id == 4639 ||
            id == 4645 ||
            id == 4651 ||
            id == 4652 ||
            id == 4679 ||
            id == 4685 ||
            id == 4691 ||
            id == 4697 ||
            id == 4699 ||
            id == 4719 ||
            id == 4722 ||
            id == 4724 ||
            id == 4730 ||
            id == 4731 ||
            id == 4749 ||
            id == 4757 ||
            id == 4762 ||
            id == 4765 ||
            id == 4773 ||
            id == 4778)
        {
            continue;
        }

        Problem problem;
        if (!loadProblem(id, false, &problem, &msg))
        {
            /*
            printf("Error loading problem %" PRIu32 ": %s\n",
                   id,
                   msg.c_str());
            */
            continue;
        }

        string solutionFileName =
            strprintf("../solutions/%" PRIu32 ".txt", id);
        string perfectFileName =
            strprintf("../perfectsolutions/%" PRIu32 ".txt", id);

#if 1
        {
            FILE* f = fopen(solutionFileName.c_str(), "rb");
            //FILE* f = fopen(perfectFileName.c_str(), "rb");
            if (f)
            {
                fclose(f);
                continue;
            }
        }
#endif

        printf("Problem %" PRIu32 "\n", id);

        Solution solution;
        string solutionData;
        uint32_t solutionSize = 0;
        uint64_t nodeCount = 0;

        bool solved = false;

#if 0
        Solution convexSolution;
        string convexSolutionData;
        uint32_t convexSolutionSize = 0;
        if (solveConvex(problem,
                        &convexSolution,
                        &convexSolutionData,
                        &convexSolutionSize) && convexSolutionSize <= 5000)
        {
            printf("*** SOLVED %" PRIu32 " CONVEX! ***\n", id);
            solution = std::move(convexSolution);
            solutionData = std::move(convexSolutionData);
            solutionSize = convexSolutionSize;
            solved = true;
        }
        else
#endif

#if 0
        Solution approxSolution;
        string approxSolutionData;
        uint32_t approxSolutionSize = 0;
        if (solveApprox(problem,
                        &approxSolution,
                        &approxSolutionData,
                        &approxSolutionSize) && approxSolutionSize <= 5000)
        {
            printf("*** SOLVED %" PRIu32 " APPROX! ***\n", id);
            solution = std::move(approxSolution);
            solutionData = std::move(approxSolutionData);
            solutionSize = approxSolutionSize;
            solved = true;
        }
        else
#endif

        {
#if 1
            solved = solve(problem,
                           seed ^ ((uint64_t)id << 17),
                           2000,
                           &solution,
                           &solutionData,
                           &solutionSize,
                           &nodeCount);
#endif
        }

        if (!solved)
        {
            printf("Not solved.  node count = %" PRIu64 "\n",
                   nodeCount);
            if (force)
            {
                id--;
                seed += 17000000;
            }
            continue;
        }

        printf("Solved.  node count = %" PRIu64 "\n",
               nodeCount);
        //printf("%s", solutionData.c_str());
        printf("solution size = %" PRIu32 "\n", solutionSize);

        if (solutionSize > 5000)
        {
            printf("Skipping submit because the size limit is exceeded\n");
            continue;
        }

#if 1
        SolutionResult result;
        if (!Protocol::submitSolution(id, solutionData, &result, &msg))
        {
            printf("%s\n", msg.c_str());
            //return 1;
            //sleepS(10);
            continue;
        }

        printf("  id = %" PRIu32 "\n", result.m_id);
        printf("  resemblance = %f\n", result.m_resemblance);
        printf("  solution hash = %s\n", result.m_solutionHash.c_str());
        printf("  solution size = %" PRIu32 "\n", result.m_solutionSize);
        //sleepMS(700);

        FILE* f = fopen(solutionFileName.c_str(), "wb");
        if (!f)
        {
            printf("Error opening solution file for write\n");
            return 1;
        }
        if (fwrite(solutionData.data(), 1, solutionData.size(), f) != solutionData.size())
        {
            fclose(f);
            printf("Error writing to solution file\n");
            return 1;
        }
        fclose(f);

        if (result.m_resemblance == 1.0)
        {
            f = fopen(perfectFileName.c_str(), "wb");
            if (!f)
            {
                printf("Error opening perfect solution file for write\n");
                return 1;
            }
            if (fwrite(solutionData.data(), 1, solutionData.size(), f) != solutionData.size())
            {
                fclose(f);
                printf("Error writing to perfect solution file\n");
                return 1;
            }
            fclose(f);
        }
#endif

#if 0
        printf("Stopping early for now\n");
        break;
#endif
    }

    return 0;
}
