#include "Common.hpp"
#include "Problem.hpp"
#include "Geometry.hpp"
#include "Solver.hpp"
#include "Challenge.hpp"
#include <gmpxx.h>

using std::string;

int main(int argc, char *argv[])
{
    string msg;

#if 0
    {
        Coord p(mpq_class("4/5"), mpq_class("3/5"));
        Coord q(mpq_class("8/5"), mpq_class("6/5"));
        Coord a(mpq_class("1/5"), mpq_class("7/5"));
        Coord b = reflect(p, q, a);
        printf("b = (%s, %s)\n",
               b.x.get_str().c_str(),
               b.y.get_str().c_str());
        return 0;
    }
#endif

    for (uint32_t id = 1; id <= 101; id++)
    {
        Problem problem;
        if (!loadProblem(id, false, &problem, &msg))
        {
            printf("Error loading problem %" PRIu32 ": %s\n",
                   id,
                   msg.c_str());
            continue;
        }

        printf("Problem %" PRIu32 "\n", id);

        for (uint32_t iPolygon = 0; iPolygon < (uint32_t)problem.m_polygons.size(); iPolygon++)
        {
            auto& polygon = problem.m_polygons[iPolygon];
            printf("  Polygon %" PRIu32 ": %s, %s\n",
                   iPolygon,
                   getPolygonDir(polygon) == Dir::CCW ? "CCW" : "CW",
                   isPolygonConvex(polygon) ? "convex" : "not convex");
            for (uint32_t iVertex = 0; iVertex < (uint32_t)polygon.m_vertices.size(); iVertex++)
            {
                auto& vertex = polygon.m_vertices[iVertex];
                printf("    Vertex %" PRIu32 ": %s,%s\n",
                       iVertex,
                       vertex.x.get_str().c_str(),
                       vertex.y.get_str().c_str());
            }
        }

        solve(problem);
    }

    /*
    mpq_class a = 2;
    mpq_class b = 6;
    mpq_class c = a / b;

    gmp_printf("c = %Qd\n", c.get_mpq_t());
    printf("c = %s\n", c.get_str().c_str());
    */

    return 0;
}
